// RELATED SOFTWARE: Scilab ≥ 6.0.0
// PACKAGE  : read_datablock : read data embeded in bloc-comments appended in your script
// VERSION  : 1.0 - 2023-05-28
// AUTHOR   : Samuel GOUGEON - Le Mans Université
// COPYRIGHT: Samuel GOUGEON -  2023
// LICENSE : CeCILL-C : https://cecill.info/licences/Licence_CeCILL-C_V1-en.html
//
// DISCLAIMER: This toolbox is provided as is, with NO WARRANTY of any kind.
//             Use it to your own risks & responsability.

function varargout = read_datablock(varargin)
    /*
    read_datablock(n)
    read_datablock(tag)

    read_datablock(filename, n)
    read_datablock(filename, tag)

    read_datablock(.., "-string")
    read_datablock(.., "-string..")

    n: integer > 0 : index of the block-comment to read
    head_tag: single string in the headline of the bc to read
    */

    fname = "read_datablock"
    filename = []
    block_id = 1
    contMarks = %t      // process continuation marks ..
    doEval = evstr
    n_in = argn(2)

    // ARGUMENTS CHECKING
    // ------------------
    if n_in == 0 | n_in > 3
        msg = _("%s: Wrong number of input arguments: %d to %d expected.\n")
        error(msprintf(msg, fname, 1, 3))
    end

    a = varargin(1)
    i = 1

    if type(a)<>1 & type(a)<>10
        msg = _("%s: Argument #%d: %s expected.\n")
        error(msprintf(msg, fname, i, _("integer or string")))
    end
    if size(a, "*") <> 1
        msg = _("%s: Argument #%d: Scalar (1 element) expected.\n")
        error(msprintf(msg, fname, i))
    end
    if type(a)==10
        if n_in==1
            block_id = a
        else
            b = varargin(2)
            if type(b) <> 10 | part(b(1),1:7)<>"-string"
                filename = a
            else
                block_id = a
            end
        end
    else    // n:
        if ~isreal(a) | a <> int(a) | a < 1
            msg = _("%s: Argument #%d: %s expected.\n")
            error(msprintf(msg, fname, i, _("integer > 0")))
        end
        block_id = a
    end

    // "-string" "-string.." ending options:
    if n_in > 1
        varargin(1) = null()
        a = varargin($)
        if type(a)==10 & size(a,"*")==1 & or(a==["-string" "-string."+"."])
            doEval = []
            if a=="-string"
                contMarks = %f
            end
            varargin($) = null()
        end
    end

    if length(varargin) > 0
        // read_datablock(filename, n)
        // read_datablock(filename, head_tag)
        i = 2
        a = varargin($)
        if type(a)<>1 & type(a)<>10
            msg = _("%s: Argument #%d: %s expected.\n")
            error(msprintf(msg, fname, i, _("integer or string")))
        end
        if size(a, "*") <> 1
            msg = _("%s: Argument #%d: Scalar (1 element) expected.\n")
            error(msprintf(msg, fname, i))
        end
        if type(a)==1 & (~isreal(a) | a <> int(a) | a < 1)
            msg = _("%s: Argument #%d: %s expected.\n")
            error(msprintf(msg, fname, i, _("integer > 0")))
        end
        block_id = a
    end

    // PROCESSING
    // ----------
    // Getting the path of the file
    // ----------------------------
    if filename == []
        [units,typ,names,mod,swap] = file()
        filename = names(typ=="C" & mod < 200)
        if size(filename,"*")>1
            // some text files might have been opened in the script before calling
            // read_datablock.
            // We get the line number of the current call to read_datablock(..):
            [Ln, ?] = where();
            Ln = Ln($)
            // Then we read each opened file: we get the file whose line #Ln includes
            //  read_datablock()
            wm = warning('query')
            warning("off")          // avoid the "file already opened" message
            for f = filename
                tmp = mgetl(f, Ln)
                if size(tmp,1)==Ln & grep(tmp($), "read_datablock(")<> []
                    filename = f
                    break
                end
            end
            warning(wm)
        end
    
        if filename==[] & or(getscilabmode()==["STD" "NW"])
            // maybe run from a selection in Scinotes
            jimport org.scilab.modules.scinotes.SciNotes
            e = SciNotes.getEditor()
            if type(e)==17  // Scinotes is actually open (otherwise ==[])
                // We get the path of the file on the foreground
                [?, ?, ?, filename] = regexp(e.toString(), "/(?:\()(.+?)(?:\))/")
            end
        end
    else
        filename = fullpath(filename)
        if ~isfile(filename)
            msg = ("%s: Argument #%d: The file ''%s'' does not exist.\n");
            error(msprintf(msg, fname, 1, filename))
        end
    end

    // Reading the file content
    wm = warning('query')
    warning("off")          // avoid the "file already opened" message
    txt = mgetl(filename)
    warning(wm)
    //
    out = []
    if type(block_id)==10
        for c = strsplit("\/{}[]()-+?,:")'
            block_id = strsubst(block_id, c, "\"+c)
        end
        ks = grep(txt, "/^\s*\/\*.*?"+block_id+"/", "r")
        block_id = 1
    else
        ks = grep(txt, "/^\s*\/\*/", "r")
    end
    ke = grep(txt, "/^\s*\*\//", "r")

    // Cleaning ks: 
    //  . "/*" inside any current block must be ignored
    //  . trailing "*/" out of opened block must be ignored
    ke2 = []
    for i = 1:length(ks)
        tmp = ke(ke > ks(i))
        if tmp <> []
            ke2 = [ke2 tmp(1)]
        else
            ks(i:$) = []    // ignore all trailing unclosed block comments
            break
        end
    end
    ke = ke2

    if ks == [] then
        varargout = list([])
        return
    else
        if or([length(ks), length(ke)] < block_id)
            varargout = list([])
            return
        end
        b_start = ks(block_id)+1
        b_end = ke(block_id)-1
    end

    if b_start<>[] then
        out = txt(b_start:b_end)
    end

    if contMarks
        // preprocess continuation marks
        out = stripblanks(out, %t, 1)
        k = grep(out, "/.*\.{2,}$/","r") // at least 2 trailing ..
        for r = k($:-1:1)
            [?,?,?,tmp] = regexp(out(r), "/(.*?)\.{2,}$/")
            out(r) = tmp + out(r+1)
            out(r+1) = []
        end
    end

    // Text-to-data conversion:
    if or(type(doEval)==[13 130])
        [T, F] = (%T, %F);
        out = doEval(out)
    end
    varargout = list(out)
endfunction

// ----------------------------------------------------------------------------
// For unit tests
/* 2nd : booleans
  F F T F F T T
  T F F T T F T
*/
/* 3rd: with ..
  -9   9   5  -2  -7 ...
  -3   4   5  -2  -4

   7   4   5   3   4 ..
  -6  -2  -1  -7   6
*/
